<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\CsvImportTrait;
use App\Http\Requests\MassDestroyCashExpenseRequest;
use App\Http\Requests\StoreCashExpenseRequest;
use App\Http\Requests\UpdateCashExpenseRequest;
use App\Models\BankAccount;
use App\Models\CashExpense;
use App\Models\FinancialCategory;
use App\Models\Header;
use App\Models\Suplier;
use App\Models\Vat;
use Gate;
use DB;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;

class CashExpensesController extends Controller
{
    use CsvImportTrait;
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->branch_id = session()->get('branch_id');
            return $next($request);
        });

    }
    public function index(Request $request)
    {
        abort_if(Gate::denies('cash_expense_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if ($request->ajax()) {
            $start =  date('Y-m-d', strtotime(strtr($request->min, '/', '-')));
            $end = date('Y-m-d', strtotime(strtr($request->max, '/', '-')));
            $query = CashExpense::query()->with(['suplier'])->where('branch_id',$this->branch_id);
            if($request->min != null || $request->max != null ){
                $query = $query->whereDate('date','>=',date('Y-m-d',strtotime($start)))->whereDate('date','<=',date('Y-m-d',strtotime($end)));
            }
            // $query = $query->select(sprintf('%s.*', (new CashExpense())->table));
            $table = Datatables::of($query);
            $table->addIndexColumn();

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->editColumn('actions', function ($row) {
                    $viewGate = 'cash_expense_show';
                    $editGate = 'cash_expense_edit';
                    $deleteGate = 'cash_expense_delete';
                    $crudRoutePart = 'cash-expenses';

                    return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                ));
            });

            $table->editColumn('date', function ($row) {
                return $row->date ? date('d/m/Y',strtotime($row->date)) : '';
            });

            $table->editColumn('invoice_no', function ($row) {
                return $row->invoice_no ? $row->invoice_no : '';
            });
            $table->addColumn('suplier', function ($row) {
                return $row->suplier ? $row->suplier->suplier : '';
            });

            $table->addColumn('bank_account', function ($row) {
                return $row->bankAccount ? $row->bankAccount->title : '';
            });

            $table->addColumn('header', function ($row) {
                $a = '<span class="badge badge-info">'.$row->cash_credit.'<span>';
                return $row->financialCategory ? $row->financialCategory->title."<br>".$a : $a;
            });

            $table->addColumn('vat', function ($row) {
                return $row->vat_value ? $row->vat_value : '';
            });

            $table->editColumn('amount', function ($row) {
                return $row->amount ? number_format($row->amount,2) : '';
            });

            $table->editColumn('amount_without_vat', function ($row) {
                return $row->amount_without_vat ? $row->amount_without_vat : '';
            });

            $table->rawColumns(['actions', 'placeholder', 'suplier','header','vat']);

            return $table->make(true);
        }

        return view('admin.cashExpenses.index');
    }

    public function create()
    {
        abort_if(Gate::denies('cash_expense_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $supliers = Suplier::where('branch_id',$this->branch_id)->where('type','Supplier')->get()->pluck('suplier', 'id')->prepend(trans('global.pleaseSelect'), '');
        $bankAccounts = BankAccount::select('id', DB::raw("CONCAT(title,' ',IFNULL(account_no,'')) AS title1"))->get()->pluck('title1','id');
        $vat = Vat::all()->where('status',1)->pluck('title', 'id')->prepend(trans('global.pleaseSelect'), '');
        $headers = FinancialCategory::with('header')
                            ->whereHas('header', function($query) {
                                $query->where('category_type','expense');})
                            ->where('status','1')->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');

        return view('admin.cashExpenses.create', compact('supliers','bankAccounts','vat','headers'));
    }

    public function store(StoreCashExpenseRequest $request)
    {
        $attach = null;
        $date1 = strtr($request->date, '/', '-');
        $date = date('Y-m-d', strtotime($date1));

        foreach ($request->input('invoice_no') as $key => $value) {
            $amount_with_vat = $request->total_amount[$key];
            $attach = '';
            $vat = null;
            $vat_value = null;
            $suplier = null;

            // Check suplier is avilable
            if($request->has('suplier')){
                foreach($request->suplier as $id => $val){
                    if($key == $id && $val != null){
                        $suplier = $request->suplier[$key];
                    }
                }
            }

            // check vat is applicable or not
            if ($request->has('vat_category')) {
                foreach ($request->vat_category as $id => $val) {
                    if ($key == $id && $val != null) {
                        $vat = $request->vat_category[$key];
                        $vat_value = Vat::find($request->vat_category[$key])->value;
                        $amount_with_vat = $request->total_amount[$key] + $request->total_amount[$key] * ($vat_value/100);
                    }
                }
            }

            // check attachement is avilable
            if($request->hasFile('attachment'))
            {
                foreach ($request->attachment as $id => $val) {
                    if ($key == $id && $val != null) {
                        if ($file = $request->file('attachment')[$key]) {
                            $optimizePath = '../public/images/cashExpense/';
                            if (!file_exists($optimizePath)) {
                                mkdir($optimizePath, 666, true);
                            }
                            $name = time() . '_cashexpense_' . $file->getClientOriginalName();
                            $file->move($optimizePath, $name);
                            $attach = $name;
                        }
                    }
                }
            }
            //  Storing data
            $cashExpense = CashExpense::create([
                'branch_id' => $this->branch_id,
                'vat_id' =>  $vat,
                'vat_value' => (double)$vat_value,
                'supplier_id' => $suplier,
                'bank_account_id' => $request->has('bank_account') ? $request->bank_account : null,
                'financial_category_id' => $request->sub_category[$key],
                'date' => $date,
                'invoice_no' => $request->invoice_no[$key],
                'cash_credit' => $request->cash_credit[$key],
                'reference' => $request->reference[$key],
                'attachment' => $attach,
                'amount' => (double)$amount_with_vat,
                'amount_without_vat' => (double)$request->total_amount[$key],
            ]);
        }
        if($request->ajax()){
            return response()->json(['success'=>true,'message'=>'Expense added successfully!','url'=> route('admin.cash-expenses.index')]);
        }
        return redirect()->route('admin.cash-expenses.index');

    }

    public function edit(CashExpense $cashExpense)
    {
        abort_if(Gate::denies('cash_expense_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $supliers = Suplier::where('branch_id',$this->branch_id)->where('type','Supplier')->get()->pluck('suplier', 'id')->prepend(trans('global.pleaseSelect'), '');
        $bankAccount = BankAccount::select('id', DB::raw("CONCAT(title,' ',IFNULL(account_no,'')) AS title1"))->get()->pluck('title1','id')->prepend(trans('global.pleaseSelect'),'');
        $vat = Vat::all()->where('status',1)->pluck('title', 'id')->prepend(trans('global.pleaseSelect'), '');
        $headers = FinancialCategory::with('header')
                            ->whereHas('header', function($query) {
                                $query->where('category_type','expense');})
                            ->where('status','1')->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');

        return view('admin.cashExpenses.edit', compact('supliers','headers','bankAccount','vat','bankAccount','cashExpense'));
    }

    public function update(UpdateCashExpenseRequest $request, CashExpense $cashExpense)
    {
        $date1 = strtr($request->date, '/', '-');
        $eff_from = date('Y-m-d', strtotime($date1));

        $amount_with_vat = $request->total_amount;
        $attach = '';
        $vat = null;
        $vat_value = null;

        if ($request->has('vat_category') && $request->vat_category != null)
        {
            $vat = $request->vat_category;
            $vat_value = Vat::find($request->vat_category)->value;
            $amount_with_vat = $request->total_amount + $request->total_amount * ($vat_value/100);
        }

        if($request->hasFile('attachment'))
            {
                if ($file = $request->file('attachment')) {
                    $optimizePath = '../public/images/cashExpense/';
                    if (!file_exists($optimizePath)) {
                        mkdir($optimizePath, 666, true);
                    }
                    $name = time() . '_cashexpense_' . $file->getClientOriginalName();
                    $file->move($optimizePath, $name);
                    $attach = $name;
                }

            }

        $cashExpense->update([
            'branch_id' => session()->get('branch_id'),
                    'vat_id' =>  $vat,
                    'vat_value' => (double)$vat_value,
                    'supplier_id' => $request->suplier,
                    'bank_account_id' => $request->bank_account,
                    'financial_category_id' => $request->sub_category,
                    'date' => $eff_from,
                    'invoice_no' => $request->invoice_no,
                    'cash_credit' => $request->cash_credit,
                    'reference' => $request->reference,
                    'attachment' => $attach,
                    'amount' => (double)$amount_with_vat,
                    'amount_without_vat' => (double)$request->total_amount,
            'description' => $request->description,
        ]);
        toastr()->info('Expense has been updated successfully!');
        return redirect()->route('admin.cash-expenses.index');
    }

    public function show(CashExpense $cashExpense)
    {
        abort_if(Gate::denies('cash_expense_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $cashExpense->load('suplier');

        return view('admin.cashExpenses.show', compact('cashExpense'));
    }

    public function destroy(CashExpense $cashExpense)
    {
        abort_if(Gate::denies('cash_expense_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $cashExpense->forceDelete();
        toastr()->error('Expense has been deleted!');
        return back();
    }

    public function massDestroy(MassDestroyCashExpenseRequest $request)
    {
        CashExpense::whereIn('id', request('ids'))->forceDelete();

        toastr()->error('Expense has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
