<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyFinancialCategoryRequest;
use App\Http\Requests\StoreFinancialCategoryRequest;
use App\Http\Requests\UpdateFinancialCategoryRequest;
use App\Models\FinancialCategory;
use App\Models\Header;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;
use Gate;

class FinancialCategoryController extends Controller
{

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->branch_id = session()->get('branch_id');
            return $next($request);
        });

    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('sub_category_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        if ($request->ajax()) {
            $query = FinancialCategory::all();
            $table = DataTables::of($query);
            $table->addIndexColumn();

            $table->addColumn('DT_RowId', function ($row) {
                return $row->id ? $row->id : '';
            });

            $table->addColumn('placeholder', '&nbsp;');
            $table->addColumn('actions', '&nbsp;');

            $table->addColumn('header',function($row){
                return $row->header ? $row->header->header_name : '';
            });

            $table->editColumn('status', function ($row) {
                return $row->status == 1 ? '<span class="badge bg-inverse-success">Active</span>' : '<span class="badge bg-inverse-danger">Inactive</span>';
            });

            $table->editColumn('actions', function ($row) {
                $viewGate = 'sub_category_show';
                $editGate = 'sub_category_edit';
                $deleteGate = '';
                if($row->type == 1){
                    $deleteGate = 'sub_category_delete';
                }
                $crudRoutePart = 'financial-categories';
                return view('partials.datatablesActions', compact(
                'viewGate',
                'editGate',
                'deleteGate',
                'crudRoutePart',
                'row'
            ));
            });

            $table->rawColumns(['status','actions', 'placeholder', 'header']);

            return $table->make(true);
        }

        return view('admin.financialCategory.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('sub_category_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $headers = Header::all()->pluck('header_name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.financialCategory.create',compact('headers'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreFinancialCategoryRequest $request)
    {
        $status = 0;
        if($request->has('status'))
        {
            $status =  (int)$request->status;
        }
        $financial = FinancialCategory::create([
            'title' => $request->financial_sub_category,
            'remarks' => $request->remarks,
            'header_id' => $request->financial_category,
            'status' => $status,
        ]);
        $financial->code = 'sub'.$financial->id;
        $financial->save();

        toastr()->success('Financial sub-category is created successfully!');
        return redirect()->route('admin.financial-categories.index');

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(FinancialCategory $financialCategory)
    {
        abort_if(Gate::denies('sub_category_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.financialCategory.show',compact('financialCategory'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit(FinancialCategory $financialCategory)
    {
        abort_if(Gate::denies('sub_category_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $headers = Header::all()->pluck('header_name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.financialCategory.edit',compact('headers','financialCategory'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateFinancialCategoryRequest $request, FinancialCategory $financialCategory)
    {
        // $status = $financialCategory->status ?? 0 ;
        $status = 0;
        if($request->has('status'))
        {
            $status =  $request->status;
        }

        $financialCategory->update([
            'title' => $request->financial_sub_category,
            'remarks' => $request->remarks,
            'header_id' => $request->financial_category,
            'status' => $status,
            'code' => 'sub'.$financialCategory->id
        ]);
        toastr()->info('Financial sub-category has been updated successfully!');
        return redirect()->route('admin.financial-categories.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(FinancialCategory $financialCategory)
    {
        abort_if(Gate::denies('sub_category_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if($financialCategory->type == '0'){
            toastr()->error('You are not allowed to delete this category!');
            return redirect()->route('admin.financial-categories.index');
        }
        if((isset($financialCategory->cashsubense) && count($financialCategory->cashsubense) > 0) || (isset($financialCategory->purchases) && count($financialCategory->purchases) > 0) || (isset($financialCategory->deposits) && count($financialCategory->deposits) > 0) || (isset($financialCategory->sales) && count($financialCategory->sales) > 0) || (isset($financialCategory->cashiersales) && count($financialCategory->cashiersales) > 0)){
            toastr()->error('It has associated data. Please delete that first!');
            return back();
        }
        $financialCategory->forceDelete();
        toastr()->error('Financial sub-category has been deleted!');
        return back();
    }

    public function massDestroy(Request $request)
    {
        abort_if(Gate::denies('sub_category_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $request->validate([
            'ids'   => 'required|array',
        ]);
        foreach($request->ids as $id){
            $financialCategory = FinancialCategory::find($id);
            $this->destroy($financialCategory);
        }

        return response(null, Response::HTTP_NO_CONTENT);
    }
}
