<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyUserRequest;
use App\Http\Requests\StoreUserRequest;
use App\Http\Requests\UpdateUserRequest;
use App\Models\AllCity;
use App\Models\AllCountry;
use App\Models\AllState;
use App\Models\Country;
use App\Models\Role;
use App\Models\SiteRegister;
use App\Models\User;
use Gate;
use Auth;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class UsersController extends Controller
{
    public function index()
    {
        abort_if(Gate::denies('user_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $users = User::with(['roles'])->get();

        return view('admin.users.index', compact('users'));
    }

    public function create()
    {
        abort_if(Gate::denies('user_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        if(Auth::user()->hasRole('Super Admin')){
            $roles = Role::all()->pluck('title', 'id');
        }
        elseif(Auth::user()->hasRole('Admin')){
            $roles = Role::where('title','!=','Super Admin')->get()->pluck('title', 'id');
        }
        else{
            $roles = Role::where('title','!=','Super Admin')->where('title','!=','Admin')->get()->pluck('title', 'id');
        }
        $branches = SiteRegister::all()->pluck('name','id')->prepend(trans('global.pleaseSelect'));
        $countries = Country::all();

        return view('admin.users.create', compact('roles','branches','countries'));
    }

    public function store(StoreUserRequest $request)
    {
        abort_if(Gate::denies('user_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $input = $request->except('_token');
        if($request->has('profile_pic') && $request->file('profile_pic') != null)
        {
            if ($file = $request->file('profile_pic')) {
                $optimizePath = '../public/images/users/';
                if (!file_exists($optimizePath)) {
                    mkdir($optimizePath, 666, true);
                }
                $name = time() . '_users_' . $file->getClientOriginalName();
                $file->move($optimizePath, $name);
                $input['profile_pic'] = $name;
            }
        }
        $input['dob'] = date('Y-m-d',strtotime(strtr($request->date,'/','-')));


        $user = User::create($input);
        $user->roles()->sync($request->input('roles', []));

        toastr()->success('User has been created!');
        return redirect()->route('admin.users.index');
    }

    public function edit(User $user)
    {
        abort_if(Gate::denies('user_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if(Auth::user()->hasRole('Super Admin')){
            $roles = Role::all()->pluck('title', 'id');
        }elseif(Auth::user()->hasRole('Admin')){
            $roles = Role::where('title','!=','Super Admin')->get()->pluck('title', 'id');
        }else{
            $roles = Role::where('title','!=','Super Admin')->where('title','!=','Admin')->get()->pluck('title', 'id');
        }

        $user->load('roles');
        $branches = SiteRegister::all();
        $countries = Country::all();

        $country = AllCountry::where('iso3',$user->country)->first();
        $states = AllState::where('country_id',$country->id)->get();
        $cities = AllCity::where('state_id',$user->state_id)->get();

        return view('admin.users.edit', compact('roles', 'user','branches','countries','states','cities'));
    }

    public function update(UpdateUserRequest $request, User $user)
    {
        abort_if(Gate::denies('user_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        if(Auth::user()->hasRole('Super Admin') || Auth::user()->hasRole('Admin')){
            $input = $request->except('_token');
            if ($file = $request->file('profile_pic')) {
                if (isset($user) && $user->profile_pic != null) {
                    $image_file = @file_get_contents('../public/images/users/' . $user->profile_pic);
                    if ($image_file) {
                        unlink('../public/images/users/' . $user->profile_pic);
                    }
                }
                $optimizePath = '../public/images/users/';
                if (!file_exists($optimizePath)) {
                    mkdir($optimizePath, 666, true);
                }
                $name = time() . '_users_' . $file->getClientOriginalName();
                $file->move($optimizePath, $name);
                $input['profile_pic'] = $name;
            }

            $input['dob'] = date('Y-m-d',strtotime(strtr($request->dob,'/','-')));

            $user->update($input);
            $user->roles()->sync($request->input('roles', []));
            toastr()->info('User has been updated!');
        }
        else{
            toastr()->error('You are not allowed to update this user!');
        }
        return redirect()->route('admin.users.index');
    }

    public function show(User $user)
    {
        abort_if(Gate::denies('user_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $user->load('roles', 'userUserAlerts');

        return view('admin.users.show', compact('user'));
    }

    public function destroy(User $user)
    {
        abort_if(Gate::denies('user_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if((Auth::user()->hasRole('Super Admin') || Auth::user()->hasRole('Admin')) && $user->id != Auth::id()){
            $user->forceDelete();
            toastr()->error('User has been deleted!');
        }else{
            toastr()->error('You are not allowed to delete this user!');
        }

        return back();
    }

    public function massDestroy(MassDestroyUserRequest $request)
    {
        User::whereIn('id', request('ids'))->forceDelete();

        toastr()->error('User has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
